@@ -287,12 +287,10 @@ def osm_post(lim,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20):
     crs_4326 = CRS("WGS84")
     crs_proj = CRS("EPSG:28992")
     transformer = Transformer.from_crs(crs_4326, crs_proj)
-    list_towers = list(towers)
-    towers_tree = KDTree(list(map(lambda x:transformer.transform(x.latlon[0],x.latlon[1]),list_towers))) if towers else None
-    m_towers = (list_towers,towers_tree,transformer)
-    list_nodes = list(nodes)
-    nodes_tree = KDTree(list(map(lambda x:transformer.transform(x.latlon[0],x.latlon[1]),list_nodes))) if nodes else None
-    m_nodes = (list_nodes,nodes_tree,transformer)
+    towers_tree = KDTree(list(map(lambda x:transformer.transform(x.latlon[0],x.latlon[1]),towers))) if towers else None
+    m_towers = (towers,towers_tree,transformer)
+    nodes_tree = KDTree(list(map(lambda x:transformer.transform(x.latlon[0],x.latlon[1]),nodes))) if nodes else None
+    m_nodes = (nodes,nodes_tree,transformer)
     end()
     #print(m_towers.size)
     #print(m_nodes.size)
@@ -321,50 +319,41 @@ def osm_post(lim,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20):
     common_end(g,file_name_out,id2types)
 
 def osm_pre(file_name_in,white=[("power","line")],black=[("power","substation")],conserve_mem=False):
-    from limic.util import start, end, status, file_size, save_pickled
+    from limic.util import start, end, status, file_size, save_pickled, split
     from xml.sax import parse
     from xml.sax.handler import ContentHandler
     from bz2 import open as bopen
     from limic.overpass import pylon
-    start("Extracting ways for power lines from",file_name_in)
-    class ways_handler(ContentHandler):
-        def __init__(self):
-            self.nodes = None
-            self.power = None
-            self.lines = []
-            self.substations = []
-            self.towers = []
-            self.type = None
-            self.id2types = {}
-        def startElement(self, name, attrs):
-            if name == 'way':
-                self.nodes = []
-                self.power = None
-            elif name == 'tag':
-                kv = (attrs['k'],attrs['v'])
-                if kv in white:
-                    self.power = "white"
-                    self.type = white.index(kv)
-                elif kv in black:
-                    self.power = "black"
-            elif name == 'nd':
-                self.nodes.append(int(attrs['ref']))
-            elif name == 'node':
-                if not conserve_mem:
-                    self.towers.append(pylon(int(attrs['id']),(float(attrs['lat']),float(attrs['lon']))))
-        def endElement(self, name):
-            if name == 'way':
-                if self.power == 'white':
-                    for node in self.nodes:
-                        self.id2types[node] = self.type
-                    self.lines.append(self.nodes)
-                elif self.power == 'black':
-                    self.substations.append(self.nodes)
-    f = bopen(file_name_in,"rb") if file_name_in[-4:] == ".bz2" else open(file_name_in,"rb")
-    ways = ways_handler()
-    parse(f, ways)
-    f.close()
-    lines, substations, id2types = ways.lines, ways.substations, ways.id2types
+    white = list(map(lambda x:(x[0],x[1][0]+'" v="'+x[1][1]+'"'),zip(range(len(white)),white)))
+    black = list(map(lambda x:(-1,x[0]+'" v="'+x[1]+'"'),black))
+    white_black = white+black
+    f = bopen(file_name_in,"rt") if file_name_in[-4:] == ".bz2" else open(file_name_in,"rt")
+    if not conserve_mem:
+        start("Reading map data from",file_name_in)
+        data = f.read()
+        f.close()
+        end()
+    start("Extracting ways for white and black")
+    if conserve_mem:
+        elems = split(f,'\n  <')
+    else:
+        elems = data.split('\n  <')
+    lines = []
+    substations = []
+    id2types = {}
+    for elem in elems:
+        if elem.startswith('way id="'):
+            tags = elem.split('\n    <tag k="')
+            for tag in tags[1:]:
+                for i,t in white_black:
+                    if tag.startswith(t):
+                        nds = list(map(lambda x:int(x.split('"')[0]),tags[0].split('\n    <nd ref="')[1:]))
+                        if i >= 0:
+                            lines.append(nds)
+                            for nd in nds:
+                                id2types[nd] = i
+                        else:
+                            substations.append(nds)
     end('')
     status(len(lines),end='   ')
     status(len(substations))
@@ -375,50 +364,34 @@ def osm_pre(file_name_in,white=[("power","line")],black=[("power","substation")]
     for substation in substations:
         for node in substation:
             id2substations.setdefault(node,[]).append(substation)
+    start("Extracting nodes for white and black")
     if conserve_mem:
-        del ways
-        start("Extracting nodes for white and black")
-        class nodes_handler(ContentHandler):
-            def __init__(self):
-                self.towers = []
-                self.nodes = set()
-                self.id2elem = {}
-            def startElement(self, name, attrs):
-                if name == 'node':
-                    i = int(attrs['id'])
-                    if i in id2lines:
-                        elem = pylon(i,(float(attrs['lat']),float(attrs['lon'])))
-                        self.towers.append(elem)
-                        self.id2elem[i] = elem
-                    elif i in id2substations:
-                        elem = pylon(i,(float(attrs['lat']),float(attrs['lon'])))
-                        self.nodes.add(elem)
-                        self.id2elem[i] = elem
-        f = bopen(file_name_in,"rb") if file_name_in[-4:] == ".bz2" else open(file_name_in,"rb")
-        ns = nodes_handler()
-        parse(f, ns)
-        f.close()
-        towers, nodes, id2elem = ns.towers, ns.nodes, ns.id2elem
-        del ns
-        end('')
-        status(len(towers),end='   ')
-        status(len(nodes))
-    else:
-        start("Extracting nodes for white and black")
-        towers = []
-        nodes = set()
-        id2elem = {}
-        for tower in ways.towers:
-            if tower.id in id2lines:
-                id2elem[tower.id] = tower
-                towers.append(tower)
-            elif tower.id in id2substations:
-                id2elem[tower.id] = tower
-                nodes.add(tower)
-        del ways
-        end()
-    start("Checking that all nodes were extracted")
+        f = bopen(file_name_in,"rt") if file_name_in[-4:] == ".bz2" else open(file_name_in,"rt")
+        elems = split(f,'\n  <')
+    towers = []
+    nodes = []
+    id2elem = {}
+    for elem in elems:
+        if elem.startswith('node id="'):
+            parts = elem.split("\n")[0].split('"')
+            if parts[-5]!=' lat=':
+                print(parts)
+            if parts[-3]!=' lon=':
+                print(parts)
+            i = int(parts[1])
+            if i in id2lines:
+                elem = pylon(i,(float(parts[-4]),float(parts[-2])))
+                towers.append(elem)
+                id2elem[i] = elem
+            if i in id2substations:
+                elem = pylon(i,(float(parts[-4]),float(parts[-2])))
+                nodes.append(elem)
+                id2elem[i] = elem
+    end('')
+    status(len(towers),end='   ')
+    status(len(nodes))
+    start("Checking that all nodes were extracted for",file_name_in)
     assert(set(id2lines.keys())==set(map(lambda x:x.id,towers)))
-    assert(set(id2substations.keys())==set(map(lambda x:x.id,list(nodes))))
+    assert(set(id2substations.keys())==set(map(lambda x:x.id,nodes)))
     end()
     return lines,substations,towers,nodes,id2elem,id2lines,id2substations,id2types
